/*******************************************************************
 * Wireless data logger
 * Copyright(c) 2013 H.Momose hirofumi@momose.com
 *******************************************************************/
#ifndef HARDWARE_PROFILE_H
#define HARDWARE_PROFILE_H

    /*******************************************************************/
    /******** USB stack hardware selection options *********************/
    /*******************************************************************/
    //This section is the set of definitions required by the MCHPFSUSB
    //  framework.  These definitions tell the firmware what mode it is
    //  running in, and where it can find the results to some information
    //  that the stack needs.
    //These definitions are required by every application developed with
    //  this revision of the MCHPFSUSB framework.  Please review each
    //  option carefully and determine which options are desired/required
    //  for your application.

    //The PICDEM FS USB Demo Board platform supports the USE_SELF_POWER_SENSE_IO
    //and USE_USB_BUS_SENSE_IO features.  Uncomment the below line(s) if
    //it is desireable to use one or both of the features.
    //#define USE_SELF_POWER_SENSE_IO
    #define tris_self_power     TRISAbits.TRISA2    // Input
    #if defined(USE_SELF_POWER_SENSE_IO)
    #define self_power          PORTAbits.RA2
    #else
    #define self_power          1
    #endif

    //#define USE_USB_BUS_SENSE_IO
    #define tris_usb_bus_sense  TRISAbits.TRISA1    // Input
    #if defined(USE_USB_BUS_SENSE_IO)
    #define USB_BUS_SENSE       PORTAbits.RA1
    #else
    #define USB_BUS_SENSE       1
    #endif


    //Uncomment the following line to make the output HEX of this
    //  project work with the MCHPUSB Bootloader
    //#define PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER

    //Uncomment the following line to make the output HEX of this
    //  project work with the HID Bootloader
    #define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

    /** Board definition ***********************************************/
    //These defintions will tell the main() function which board is
    //  currently selected.  This will allow the application to add
    //  the correct configuration bits as wells use the correct
    //  initialization functions for the board.  These defitions are only
    //  required in the stack provided demos.  They are not required in
    //  final application design.
    #define CLOCK_FREQ 48000000
    #define _XTAL_FREQ CLOCK_FREQ
    #define GetSystemClock() (CLOCK_FREQ)


    /** LED ************************************************************/
    #define mInitAllLEDs()      LATE &= 0xFC; TRISE &= 0xFC;
    #define mLED_1              LATEbits.LATE0
    #define mLED_2              LATEbits.LATE1
    #define mGetLED_1()         mLED_1
    #define mGetLED_2()         mLED_2
    #define mLED_1_On()         mLED_1 = 1;
    #define mLED_2_On()         mLED_2 = 1;
    #define mLED_1_Off()        mLED_1 = 0;
    #define mLED_2_Off()        mLED_2 = 0;
    #define mLED_1_Toggle()     mLED_1 = !mLED_1;
    #define mLED_2_Toggle()     mLED_2 = !mLED_2;

    /** SWITCH *********************************************************/
    #define mInitAllSwitches()  TRISB |= 0x0F;
    #define mInitSwitch1()      TRISBbits.TRISB4=1;
    #define mInitSwitch2()      TRISBbits.TRISB5=1;
    #define mInitSwitch3()      TRISBbits.TRISB6=1;
    #define mInitSwitch4()      TRISBbits.TRISB7=1;
    #define sw1			PORTBbits.RB4
    #define sw2                 PORTBbits.RB5
    #define sw3                 PORTBbits.RB6
    #define sw4                 PORTBbits.RB7

    /** Graphic LCD ****************************************************/
    #define GLCD_RS             PORTAbits.RA0
    #define GLCD_RW             PORTAbits.RA1
    #define GLCD_EN             PORTAbits.RA2
    #define GLCD_CL             PORTAbits.RA5
    #define GLCD_DB             PORTD
    #define GLCD_DBO            LATD
    #define GLCD_DBIN()         TRISD = 0xFF;
    #define GLCD_DBOUT()        TRISD = 0x00;
    #define GLCD_BL             PORTCbits.RC2

    /** SD Card ********************************************************/
    #define SD_CSEL             PORTEbits.RE2
    #define SD_SDOU             PORTCbits.RC7
    #define SD_SDIN             PORTBbits.RB0
    #define SD_SCLK             PORTBbits.RB1
    #define SD_CDET             (!PORTBbits.RB3)
    #define SD_WPRT             PORTCbits.RC6

    /** LED & Buzzer ***************************************************/
    #define LED1                PORTEbits.RE0
    #define LED2                PORTEbits.RE1
    #define BUZZER              PORTCbits.RC1

    /** Radio data *****************************************************/
    #define RXIN                PORTBbits.RB2

    /** USB external transceiver interface (optional) ******************/
    #define tris_usb_vpo        TRISBbits.TRISB3    // Output
    #define tris_usb_vmo        TRISBbits.TRISB2    // Output
    #define tris_usb_rcv        TRISAbits.TRISA4    // Input
    #define tris_usb_vp         TRISCbits.TRISC5    // Input
    #define tris_usb_vm         TRISCbits.TRISC4    // Input
    #define tris_usb_oe         TRISCbits.TRISC1    // Output
    #define tris_usb_suspnd     TRISAbits.TRISA3    // Output

    /** I/O pin definitions ********************************************/
    #define INPUT_PIN 1
    #define OUTPUT_PIN 0

    //These definitions are only relevant if the respective functions are enabled
    //in the usb_config.h file.
    //Make sure these definitions match the GPIO pins being used for your hardware
    //setup.
    #define UART_DTS PORTBbits.RB4
    #define UART_DTR LATDbits.LATD3
    #define UART_RTS LATAbits.LATA2
    #define UART_CTS PORTAbits.RA3

    #define mInitRTSPin() {TRISAbits.TRISA2 = 0;}   //Configure RTS as a digital output.
    #define mInitCTSPin() {TRISAbits.TRISA3 = 1;}   //Configure CTS as a digital input.  (Make sure pin is digital if ANxx functions is present on the pin)
    #define mInitDTSPin() {TRISBbits.TRISB4 = 1;}   //Configure DTS as a digital input.  (Make sure pin is digital if ANxx functions is present on the pin)
    #define mInitDTRPin() {TRISDbits.TRISD3 = 0;}   //Configure DTR as a digital output.

#endif  //HARDWARE_PROFILE_H
