/******************************************************************************/
/*  Wireless Temp/Humidity Logger                                             */
/*  Reciever unit lcd control                                                 */
/*  Copyright(c) 2013 Hirofumi Momose  hirofumi[at]momose.com                 */
/*  Based on software from picfun.com                                         */
/******************************************************************************/

#include "HardwareProfile.h"
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "glcd.h"
#include "font.h"
#include "delays.h"

// Clock 48MHz, 1/4 => 12MHz tcy = 1/12MHz => 83.3ns
// #define LCD_WAIT_1us() {Delay10TCY(); Delay1TCY(); Delay1TCY();}
#define LCD_WAIT_1us() __delay_us(1)

#define RS_CMD 0
#define RS_DAT 1
#define RW_RED 1
#define RW_WRT 0
#define EN_ENA 1
#define EN_DIS 0

void glcd_backlight(char bright)
{
    PR2 = 0xff;
    T2CONbits.T2CKPS = 2;   // Use 1/16 prescaler
    T2CONbits.TMR2ON = 1;
    CCP1CONbits.CCP1M = 0x0c;
    
    switch(bright){
        case 0: CCPR1L = 0; break;
        case 1: CCPR1L = 28; break;
        case 2: CCPR1L = 57; break;
        case 3: CCPR1L = 85; break;
        case 4: CCPR1L = 114; break;
        case 5: CCPR1L = 142; break;
        case 6: CCPR1L = 171; break;
        case 7: CCPR1L = 199; break;
        case 8: CCPR1L = 228; break;
        case 9: default: CCPR1L = 255; break;
    }

    CCP1CONbits.DC1B = 0;   // LSB(1:0) of duty
}

void glcd_cmd(char cmd)
{
    GLCD_DBOUT();
    GLCD_RW = RW_WRT;
    GLCD_DBO = cmd;     // need more than 100ns. hold 10 ns.
    GLCD_RS = RS_CMD;

    LCD_WAIT_1us();
    GLCD_EN = EN_ENA;
    LCD_WAIT_1us();
    GLCD_EN = EN_DIS;   // tcy = 83ns thus no wait required
    GLCD_RW = RW_RED;
}

void glcd_write(char data)
{
    GLCD_DBOUT();
    GLCD_RW = RW_WRT;
    GLCD_DBO = data;
    GLCD_RS = RS_DAT;
    LCD_WAIT_1us();
    GLCD_EN = EN_ENA;
    LCD_WAIT_1us();
    GLCD_EN = EN_DIS;
    GLCD_RW = RW_RED;
}

char glcd_read(void)
{
    char data =0;
    GLCD_DBIN();
    GLCD_RW = RW_RED;
    GLCD_RS = RS_DAT;
    LCD_WAIT_1us();
    GLCD_EN = EN_ENA;
    LCD_WAIT_1us();
    data = GLCD_DB;
    GLCD_EN = EN_DIS;
    LCD_WAIT_1us();
    GLCD_DBOUT();
    return data;
}

void glcd_init(void)
{
    // Set port direction
    TRISAbits.TRISA0 = 0;	// RS control
    TRISAbits.TRISA1 = 0;	// RW control
    TRISAbits.TRISA2 = 0;	// EN control
    TRISAbits.TRISA3 = 0;	// CS1 control(not used)
    TRISAbits.TRISA4 = 0;	// CS2 control(not used)
    TRISAbits.TRISA5 = 0;	// CL control
    TRISCbits.TRISC2 = 0;	// Backlight

    PORTEbits.RDPU  =  0;       // Enable PORTD pull-ups

    // LCD Reset
    LATAbits.LATA5 = 0;
    // wait for 1ms
    //wait_ms(10);
    __delay_ms(10);
    LATAbits.LATA5 = 1;
    // wait for 1ms
    //wait_ms(10);
    __delay_ms(10);
    
    /* Initialization code from http://avr.paslog.jp/article/1252802.html */
    glcd_cmd(0xaf); // LCD On
    glcd_cmd(0x23);
    glcd_cmd(0x2f); // Set power control mode
    glcd_cmd(0x81); // Set electronic volume mode
    glcd_cmd(0x05); // *[0a-0f]set volume data 0x0000~0x003f
    glcd_cmd(0x24); // *[0x24]Set V5 internal voltage adjustment resistor
    glcd_cmd(0xa3); // ***[0xa3 only]Set LCD bias
    glcd_cmd(0xc8); // Com scan direction set, reverse
    glcd_cmd(0xa0); // normal, Select segment direction, 
    glcd_cmd(0xa4); // normal, Set all point ON/OFF command
    glcd_cmd(0xa6); // normal, Set normal/reverse display command
    glcd_cmd(0xad); // normal, sleep mode ,Close static indicator
    glcd_cmd(0x00); // Command data
    glcd_cmd(0x40 +32); // Set start common RAM
    //glcd_cmd(0xe0);  // Set read/write mode	
}

void glcd_clear(char data)
{
    unsigned char page, column;

    for(page=0; page<8; page++){			// repeat 8 page
        glcd_cmd(0xB0 +page);				// page set
        glcd_cmd(0x10);						// column set to 0
        glcd_cmd(0x01);
        for(column=0; column<128; column++){	// repeat 128 column
            glcd_write(data);
        }
    }
    // lcd_cmd(0xC0);							// reset start line
}

/****************************
*  Draw Piccel Function
*  W(0,0)-(127,63)  
*****************************/
void glcd_pixel(int Xpos, int Ypos, char On)
{
	char data, page, count, pos;

	/* if colum >127 then do nothing  */
	if(Xpos<128){
		// Y address set
		page = (char)(7-Ypos/8);			// set page
		glcd_cmd(0xB0 +page);
		
		// X address set(low)
		glcd_cmd(0x00 + (Xpos & 0x0F));
		// X address set(high)
		glcd_cmd(0x10 + ((Xpos >> 4) & 0x0F));
		
		// read current data
                glcd_read();    // dummy read is required
                data = glcd_read();
                pos =1;				// set bit position
		count = (char)(7-Ypos%8);       // set bit
                switch(count){
                    case 0: pos = 1; break;
                    case 1: pos = 2; break;
                    case 2: pos = 4; break;
                    case 3: pos = 8; break;
                    case 4: pos = 16; break;
                    case 5: pos = 32; break;
                    case 6: pos = 64; break;
                    case 7: pos = 128; break;
                }
		
		// back address
		// X set(low)
		glcd_cmd(0x00 + (Xpos & 0x0F));
		// X set(high)
		glcd_cmd(0x10 + ((Xpos >> 4) & 0x0F));

		if(On==1)							// set or reset bit
			glcd_write(data | pos);
		else
			glcd_write(data & ~pos);
	}
}

/***************************
*  `֐
***************************/
#define	abs(a)	(((a)>0) ? (a) : -(a))
void glcd_line(int x0, int y0, int x1, int y1)
{
	int steep, t;
	int	deltax, deltay, error;
	int x, y;
	int ystep;
	
	/// ̑傫ق߂
	steep = (abs(y1 - y0) > abs(x1 - x0));
	/// A̓ւ
	if(steep){
		t = x0; x0 = y0; y0 = t;
		t = x1; x1 = y1; y1 = t;
	}
	if(x0 > x1) {
		t = x0; x0 = x1; x1 = t;
		t = y0; y0 = y1; y1 = t;
	}
	deltax = x1 - x0;						// XvZ	
	deltay = abs(y1 - y0);
	error = 0;
	y = y0;
	/// XŃXebv̐؂ւ
	if(y0 < y1) ystep = 1; else ystep = -1;
	/// _ŕ`
	for(x=x0; x<x1; x++) {
		if(steep) glcd_pixel(y,x,1); else glcd_pixel(x,y,1);
		error += deltay;
		if((error << 1) >= deltax) {
			y += ystep;
			error -= deltax;
		}
	}
}

/*************************
*  \֐
*  (0, 0) - (7, 15)
**************************/
#if 0
void glcd_char(char line, char column, int letter)
{
	char i;
	int pos;

	if(column < 16){
		glcd_cmd(0xB0 + line);	// set page
		
		pos = column * 8;
		// X address set(low)
		glcd_cmd(0x00 + (pos & 0x0F));
		// X address set(high)
		glcd_cmd(0x10 + ((pos >> 4) & 0x0F));

                // Write character
		for(i=0; i<5; i++){
			glcd_write(Font[letter-0x20][i]);
		}
                // Write space
		glcd_write(0);
		glcd_write(0);
		glcd_write(0);
	}
}
#endif
/*************************
*  \֐2 7x8
*  (0, 0) - (7, 17)
**************************/
#if 0
void glcd_char1(char line, char colum, int letter)
{
	char cs, i;
	int pos;
	  
	if(colum < 18){
		glcd_cmd(0xB8 +line);		// set page
		pos = colum * 7;
		// X address set(low)
		glcd_cmd(0x00 + (pos & 0x0F));
		// X address set(high)
		glcd_cmd(0x10 + ((pos >> 4) & 0x0F));
                // Write character
		for(i=0; i<5; i++){
			glcd_write(Font[letter-0x20][i]);
		}
		// Write space
                glcd_write(0);
		glcd_write(0);
	}
}
#endif
/******************************
*   `֐
******************************/
#if 0
void glcd_str(char line, char column, char *s)
{
    while (*s)
        glcd_char1(line, column++, *s++);
}
#endif

static char row_saved, col_saved;
void glcd_cursor(char row, char col)
{
    unsigned char pos;
    row_saved = row;
    col_saved = col;
    glcd_cmd(0xB0 + row);                   // Page set
    pos = col *6 +1;
    glcd_cmd(0x00 + (pos & 0x0F));          // X set (low)
    glcd_cmd(0x10 + ((pos >> 4) & 0x0F));   // X set (high)
}

void glcd_putc(char letter)
{
    char i;
    for(i=0; i<5; i++){
        glcd_write(Font[letter - 0x20][i]);
    }
    // Write spacer
    glcd_write(0);
    // glcd_write(0);
}

void glcd_putc16(char letter)
{
    unsigned char i, pos, c;

    switch(letter){
        
        case '-': c = 10; break;
        case '.': c = 11; break;
        case '%': c = 12; break;
        case 'C': c = 13; break;
        case ' ': c = 14; break;
        default: c = letter - 0x30; break;
    }

    glcd_cmd(0xB0 + row_saved);             // Page set
    pos = col_saved *8 +1;
    glcd_cmd(0x00 + (pos & 0x0F));          // X set (low)
    glcd_cmd(0x10 + ((pos >> 4) & 0x0F));   // X set (high)
    
    // Display uppwer
    for(i=0; i<8; i++){
        glcd_write(Font16[c][i]);
    }

    glcd_cmd(0xB0 + row_saved +1);          // Page set
    pos = col_saved++ *8 +1;
    glcd_cmd(0x00 + (pos & 0x0F));          // X set (low)
    glcd_cmd(0x10 + ((pos >> 4) & 0x0F));   // X set (high)

    // Display lower
    for(i=8; i<16; i++){
        glcd_write(Font16[c][i]);
    }
}

void glcd_puts16(const char *str)
{
    while(*str){
        glcd_putc16(*str++);
    }
}

void glcd_puts(const char *str)
{
    while(*str){
        glcd_putc(*str++);
    }
}

void glcd_reverse(char row, char col, char width)
{
    unsigned char pos, pix_width, pix_count, data;
    pos = col *6 +1;
    pix_width = width * 6 -1;

    // Start read-modity-write
    glcd_cmd(0xE0);

    // Set initial position
    glcd_cmd(0xB0 + row);                   // Page set
    glcd_cmd(0x00 + (pos & 0x0F));          // X set (low)
    glcd_cmd(0x10 + ((pos >> 4) & 0x0F));   // X set (high)

    // Reverse display
    for(pix_count=0; pix_count < pix_width; pix_count++){

        // dummy read
        glcd_read();

        // read data
        data = glcd_read();

        // reverse data
        data ^= 0xff;

        // write data
        glcd_write(data);
    }

    // ENnd read-modify_write
    glcd_cmd(0xEE);
}

void glcd_puticon(char icon)
{
    char i;
    for(i=0; i<8; i++){
        glcd_write(Icon8x8[icon][i]);
    }
}

#if 0
/*****************************
* C[W\֐
*****************************/
void glcd_image(char *ptr)
{
	char cs, Xpos;
	int page, colum;

	for(page=0; page<8; page++){
		for(colum=0; colum<128; colum++){
			glcd_cmd(0xB8+page);

			// X address set(low)
			glcd_cmd(0x00 + (Xpos & 0x0F));
			// X address set(high)
			glcd_cmd(0x10 + ((Xpos >> 4) & 0x0F));

			glcd_write(*ptr++);
		}
	}
}
#endif
// EOF
