/******************************************************************************/
/*  Wireless Temp/Humidity Logger                                             */
/*  Reciever unit main                                                        */
/*  Copyright(c) 2013 Hirofumi Momose  hirofumi[at]momose.com                 */
/******************************************************************************/

/** INCLUDES *******************************************************/
#include <xc.h>
#include "Compiler.h"
#include "HardwareProfile.h"
#include "USB/usb.h"
#include "usb_function_cdc.h"
#include "rx.h"
#include "fs.h"
#include "daytime.h"
#include "glcd.h"
#include "tiny_printf.h"

/** CONFIGURATION **************************************************/
    #pragma config PLLDIV   = 2         // Use 8 MHz crystal
    #pragma config CPUDIV   = OSC1_PLL2
    #pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
    #pragma config FOSC     = HSPLL_HS
    #pragma config FCMEN    = OFF       // Fail safe clock disabled
    #pragma config IESO     = OFF       // Int / Ext clock switchover
    #pragma config PWRT     = OFF       // Power up timer
    #pragma config BOR      = OFF       // Brown out reset
    #pragma config BORV     = 3
    #pragma config VREGEN   = OFF       //USB Voltage Regulator
    #pragma config WDT      = OFF
    #pragma config WDTPS    = 32768
    #pragma config CCP2MX   = ON
    #pragma config PBADEN   = OFF       // Port B A/D Enabler
    #pragma config LPT1OSC  = OFF       // Low power timer1 osc Enabler
    #pragma config MCLRE    = ON
    #pragma config STVREN   = ON        // Stack over/under reset enabler
    #pragma config LVP      = OFF
    #pragma config ICPRT    = OFF       // Dedicated In-Circuit Debug/Programming
    #pragma config XINST    = OFF       // Extended Instruction Set
    #pragma config DEBUG    = OFF
//  #pragma config CP0      = OFF
//  #pragma config CP1      = OFF
//  #pragma config CP2      = OFF
//  #pragma config CP3      = OFF
//  #pragma config CPB      = OFF
//  #pragma config CPD      = OFF
//  #pragma config WRT0     = OFF
//  #pragma config WRT1     = OFF
//  #pragma config WRT2     = OFF
//  #pragma config WRT3     = OFF
//  #pragma config WRTB     = OFF       // Boot Block Write Protection
//  #pragma config WRTC     = OFF
//  #pragma config WRTD     = OFF
//  #pragma config EBTR0    = OFF
//  #pragma config EBTR1    = OFF
//  #pragma config EBTR2    = OFF
//  #pragma config EBTR3    = OFF
//  #pragma config EBTRB    = OFF

#define NUM_ID    10

// Current data
struct curr_data_st {
    unsigned char vbatt;            // Battery voltage
    signed short humid;             // Humidity
    signed short temp;              // Temperature
    unsigned long rx_epoch;         // Rx epoch
    unsigned char rx_hour, rx_min;  // Rx time
} curr_data[NUM_ID];

// Label
char label[NUM_ID][8];

// Alarm data
struct alarm_data_st {
    union {
        struct {
            unsigned thi_state: 2;
            unsigned tlo_state: 2;
            unsigned hhi_state: 2;
            unsigned hlo_state: 2;
        };
        unsigned char states;
    };
    signed char temp_hi, temp_lo, humid_hi, humid_lo;
} alarm_data[NUM_ID];

#define ALM_OFF 0
#define ALM_ARMED 1
#define ALM_TRIGD 2
#define ALM_MUTE  3

unsigned char backlight = 5;
unsigned char usb_talker = 0;
unsigned long rx_epoch = 0;
/** E E P R O M  *************************************************************/

/* EEPROM ADDR
 * 0 - 39 : Alarm setting
 * 0 : Tmp_hi
 * 1 : Tmp_lo
 * 2 : Humid_hi
 * 3 : Humid_lo
 */

#define EEADR_ALM_START  0
#define EEADR_ALM_END   39
#define EEADR_TMP_HI_OFS 0
#define EEADR_TMP_LO_OFS 1
#define EEADR_HUM_HI_OFS 2
#define EEADR_HUM_LO_OFS 3
#define EEADR_LCD       40
#define EEADR_USB_TALKER 41

#define EEADR_LABEL_START 48
#define EEADR_LABEL_0 48
#define EEADR_LABEL_1 56
#define EEADR_LABEL_2 64
#define EEADR_LABEL_3 72
#define EEADR_LABEL_4 80
#define EEADR_LABEL_5 88
#define EEADR_LABEL_6 96
#define EEADR_LABEL_7 104
#define EEADR_LABEL_8 112
#define EEADR_LABEL_9 120
#define EEADR_LABEL_END 120

            /* THi,  TLo,  HHi,  HLo,  THi,  TLo,  HHi,  HLo  */
__EEPROM_DATA(0x1e, 0x00, 0x50, 0x14, 0x1e, 0x00, 0x50, 0x14); /* 0 1 */
__EEPROM_DATA(0x1e, 0x00, 0x50, 0x14, 0x1e, 0x00, 0x50, 0x14); /* 2 3 */
__EEPROM_DATA(0x1e, 0x00, 0x50, 0x14, 0x1e, 0x00, 0x50, 0x14); /* 4 5 */
__EEPROM_DATA(0x1e, 0x00, 0x50, 0x14, 0x1e, 0x00, 0x50, 0x14); /* 6 7 */
__EEPROM_DATA(0x1e, 0x00, 0x50, 0x14, 0x1e, 0x00, 0x50, 0x14); /* 8 9 */
__EEPROM_DATA(0x05, 0x01, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff); /* LCD, USB_TALKER */

//__EEPROM_DATA('I', 'D', '0', ' ', ' ', ' ', ' ', ' '); /* Default Label 0*/
__EEPROM_DATA( 'S', 'h',   'o',  's',  'a',  'i', ' ', ' ');
//__EEPROM_DATA('I', 'D', '1', ' ', ' ', ' ', ' ', ' '); /* Default Label 1*/
__EEPROM_DATA( 'L', 'i',   'v',  'i',  'n',  'g', ' ', ' ');
//__EEPROM_DATA('I', 'D', '2', ' ', ' ', ' ', ' ', ' '); /* Default Label 2*/
__EEPROM_DATA( 'B', 'e',   'd',  'r',  'm',  ' ', ' ', ' ');
//__EEPROM_DATA('I', 'D', '3', ' ', ' ', ' ', ' ', ' '); /* Default Label 3*/
__EEPROM_DATA( 'C', 'a',   'b',  'i',  'n',  'e', ' ', ' ');
//__EEPROM_DATA('I', 'D', '4', ' ', ' ', ' ', ' ', ' '); /* Default Label 4*/
__EEPROM_DATA( 'W', 'a',   's',  'i',  't',  's', ' ', ' ');
//__EEPROM_DATA('I', 'D', '5', ' ', ' ', ' ', ' ', ' '); /* Default Label 5*/
__EEPROM_DATA( 'N', 'a',   'n',  'd',  'o',  ' ', ' ', ' ');
__EEPROM_DATA('I', 'D', '6', ' ', ' ', ' ', ' ', ' '); /* Default Label 6*/
__EEPROM_DATA('I', 'D', '7', ' ', ' ', ' ', ' ', ' '); /* Default Label 7*/
__EEPROM_DATA('I', 'D', '8', ' ', ' ', ' ', ' ', ' '); /* Default Label 8*/
__EEPROM_DATA('I', 'D', '9', ' ', ' ', ' ', ' ', ' '); /* Default Label 9*/



/** V A R I A B L E S ********************************************************/

#define SYS_FREQ        48000000L
#define FCY             SYS_FREQ/4

#define TMR0CLK         (FCY/16)
#define MSEC2TMR0(x)    (TMR0CLK/1000*(x))

#define TMR1CLK         (FCY/8)
#define MSEC2TMR1(x)    (TMR1CLK/1000*(x))


/** P R I V A T E  P R O T O T Y P E S ***************************************/
void screen_main(unsigned char);
void screen_setup(unsigned char);
void screen_rxstats(unsigned char);
void sw1_push();
void sw2_push();
void sw3_push();
void sw4_push();
void display();
void alarm_x100ms(void);

/******************************************************************************/
/* Interrupt / Tick counter                                                   */
/******************************************************************************/
unsigned char xtsk_25ms, xtsk_500ms, xtsk_1m;
unsigned long cnt25ms, cnt1s;
void interrupt isr(void)
{
    if(INTCONbits.TMR0IF){
        // 25ms periodical timer interrupt
        TMR0 = 0x10000 - MSEC2TMR0(25) +2;  // +2 clock adjust
        INTCONbits.TMR0IF = 0;
        
        if(cnt25ms %4 == 0){
            alarm_x100ms();
        }

        if(cnt25ms %20 == 0){
            xtsk_500ms = 1;
        }
        if(cnt25ms %40 == 0){
            daytime_update();
            if(cnt1s %60 == 0) xtsk_1m = 1;
            cnt1s++;
        }
        xtsk_25ms = 1;
        cnt25ms++;

    } else {
        // Other interupts must be invoked by data receiver
        rx_int();
    }
}


/******************************************************************************/
/* SD card control / logger                                                   */
/******************************************************************************/
enum sd_state_e {
    SD_NONE,
    SD_PROTECTED,
    SD_READY,
    SD_BUSY,
    SD_ERROR
} sd_state;

const rom char *sd_msg[] = {"None     ", "Protected", "Ready    ", "Busy     ", "Error    "};
unsigned char fs_lasterr;

void check_card(void)
{
    if(!SD_CDET){
        sd_state = SD_NONE;
    } else {
        if(SD_WPRT){
            sd_state = SD_PROTECTED;
        } else {
            if(sd_state == SD_NONE){
                fs_lasterr = fs_init();
                if(fs_lasterr == FS_OK){
                    sd_state = SD_READY;
                } else {
                    sd_state = SD_ERROR;
                }
            }
        }
    }
}

unsigned char log_day;
void log_start(void)
{
    char fname[16];

    if(sd_state == SD_READY){
        tiny_printf(NULL, fname, "%02d%02d%02d.csv", daytime.year, daytime.mon, daytime.day);
        fs_lasterr = fs_open(fname, FS_MODE_AP);
        if(fs_lasterr == FS_OK){
            log_day = daytime.day;
            sd_state = SD_BUSY;
        }
    }
}

void log_stop(void)
{
    if(sd_state == SD_BUSY){
        fs_close();
        sd_state = SD_READY;
    }
}

// output data format
// 2013/02/01 12:34:56,id1_temp,id1_humidity,id1_vbatt, ...
void log_record(void)
{
    char i;
    char buff[32];
    if(sd_state == SD_BUSY){
        if(daytime.day != log_day){
            // Rotate log
            log_stop();
            log_start();
        }

        // timestamp
        tiny_printf(NULL, buff, "%04d/%02d/%02d %02d:%02d:%02d",
                daytime.year +2000, daytime.mon, daytime.day,
                daytime.hour, daytime.min, daytime.sec);

        fs_lasterr = fs_write(buff, strlen(buff));
        if(fs_lasterr){
            sd_state = SD_ERROR;
            return;
        }

        // write all data
        for(i=0; i<10; i++){
            if(epoch - curr_data[i].rx_epoch > 600){
                tiny_printf(NULL, buff, ",,,");
            } else {
                tiny_printf(NULL, buff, ",%d.%01d,%d.%01d,%01d.%01d",
                    curr_data[i].temp/10,  curr_data[i].temp%10,
                    curr_data[i].humid/10, curr_data[i].humid%10,
                    curr_data[i].vbatt/10, curr_data[i].vbatt%10);
            }
            fs_lasterr = fs_write(buff, strlen(buff));
            if(fs_lasterr){
                sd_state = SD_ERROR;
                return;
            }
        }

        // Write CRLF
        fs_lasterr = fs_write("\r\n", 2);
        if(fs_lasterr){
            sd_state = SD_ERROR;
         }
    }
}

/******************************************************************************/
/* Beeper / Alarm                                                             */
/******************************************************************************/
void beep(unsigned char ison)
{
    PR2 = 0xff;
    T2CONbits.T2CKPS = 2;       // Use 1/16 prescaler
    T2CONbits.TMR2ON = 1;
    CCP2CONbits.CCP2M = 0x0c;
    CCP2CONbits.DC2B = 0;       // LSB(1:0) of duty
    CCPR2L = ison ? 0x80 : 0;   // MSB(9:2) of duty
}

unsigned char sound_rythm, sound_ison;
void alarm_x100ms(void)
{
    if(sound_ison){
        switch(sound_rythm & 0x0f){
            case 0: beep(1); break;
            case 2: beep(1); break;
            default: beep(0); break;
        }
    } else {
        beep(0);
    }
    sound_rythm++;
}

#define sound_on()  (sound_ison =1)
#define sound_off() (sound_ison =0)

void alarm_mute(void)
{
    unsigned char i;
    for(i=0; i<NUM_ID; i++){
        if(alarm_data[i].thi_state == ALM_TRIGD){
            alarm_data[i].thi_state == ALM_MUTE;
        }
        if(alarm_data[i].tlo_state == ALM_TRIGD){
            alarm_data[i].tlo_state == ALM_MUTE;
        }
        if(alarm_data[i].hhi_state == ALM_TRIGD){
            alarm_data[i].hhi_state == ALM_MUTE;
        }
        if(alarm_data[i].hlo_state == ALM_TRIGD){
            alarm_data[i].hlo_state == ALM_MUTE;
        }
    }
    sound_off();
}

void alarm_check(unsigned char id, struct curr_data_st *cdp)
{
    // Alarm (temp_hi)
    if(cdp->temp/10 > alarm_data[id].temp_hi){
        if(alarm_data[id].thi_state == ALM_ARMED){
            alarm_data[id].thi_state = ALM_TRIGD;
            sound_on();
        }
    } else {
        if(alarm_data[id].thi_state != ALM_OFF){
            alarm_data[id].thi_state = ALM_ARMED;
        }
    }
    // Alarm (temp_lo)
    if(cdp->temp/10 < alarm_data[id].temp_lo){
        if(alarm_data[id].tlo_state == ALM_ARMED){
            alarm_data[id].tlo_state = ALM_TRIGD;
            sound_on();
        }
    } else {
        if(alarm_data[id].tlo_state != ALM_OFF){
            alarm_data[id].tlo_state = ALM_ARMED;
        }
    }
    // Alarm (humid_hi)
    if(cdp->humid/10 > alarm_data[id].humid_hi){
        if(alarm_data[id].hhi_state == ALM_ARMED){
            alarm_data[id].hhi_state = ALM_TRIGD;
            sound_on();
        }
    } else {
        if(alarm_data[id].hhi_state != ALM_OFF){
            alarm_data[id].hhi_state = ALM_ARMED;
        }
    }
    // Alarm (humid_lo)
    if(cdp->humid/10 < alarm_data[id].humid_lo){
        if(alarm_data[id].hlo_state == ALM_ARMED){
            alarm_data[id].hlo_state = ALM_TRIGD;
            sound_on();
        }
    } else {
        if(alarm_data[id].hlo_state != ALM_OFF){
            alarm_data[id].hlo_state = ALM_ARMED;
        }
    }
}

/******************************************************************************/
/* USB Talker                                                                 */
/******************************************************************************/
void talk_to_host(unsigned char id, struct curr_data_st *cdp)
{
    char buff[64];
    short temp = cdp->temp;
    if(temp < 0){
        temp = -temp;
    }

    tiny_printf(NULL, buff, "%04d/%02d/%02d %02d:%02d:%02d,ID=%d,T=%s%d.%01d,H=%d.%01d,V=%01d.%01d\r\n",
        daytime.year +2000, daytime.mon, daytime.day, daytime.hour, daytime.min, daytime.sec,
        id, (cdp->temp < 0) ? "-":"", temp/10, temp%10, cdp->humid/10, cdp->humid%10, cdp->vbatt/10, cdp->vbatt%10);
    putsUSBUSART(buff);
}


/******************************************************************************/
/* Main                                                                       */
/******************************************************************************/
void main(void)
{
    union data_u rxdata;
    unsigned char sw, swo, swcnt;
    unsigned char i, j;

    // Port A init
    ADCON1 = 0x0F;          // Default all pins to digital

    // Setup 25ms task timer (Use TMR0)
    T0CONbits.T08BIT = 0;   // 16bit
    T0CONbits.T0CS = 0;     // Use internal clock
    //T0CONbits.T0PS = 4;   // 1/32
    T0CONbits.T0PS = 3;     // 1/16
    T0CONbits.PSA = 0;      // Use prescaler
    T0CONbits.TMR0ON = 1;
    INTCONbits.TMR0IE = 1;

    // Port E setup (led output)
    TRISEbits.RE0 = 0;
    TRISEbits.RE1 = 0;

    // PortB setup (sw input)
    TRISBbits.TRISB4 = 1;
    TRISBbits.TRISB5 = 1;
    TRISBbits.TRISB6 = 1;
    TRISBbits.TRISB7 = 1;

    // Port C Setup
    TRISCbits.TRISC1 = 0;   // Beeper

    INTCON2bits.RBPU = 1;
    TRISE = 0x00;

    // Load alarm setting from eeprom
    for(i=0; i<NUM_ID; i++){
        alarm_data[i].temp_hi = eeprom_read(i*4 +EEADR_TMP_HI_OFS);
        alarm_data[i].temp_lo = eeprom_read(i*4 +EEADR_TMP_LO_OFS);
        alarm_data[i].humid_hi = eeprom_read(i*4 +EEADR_HUM_HI_OFS);
        alarm_data[i].humid_lo = eeprom_read(i*4 +EEADR_HUM_LO_OFS);
        if(alarm_data[i].humid_lo & 0x80){
            // Alarm on
            alarm_data[i].thi_state = ALM_ARMED;
            alarm_data[i].tlo_state = ALM_ARMED;
            alarm_data[i].hhi_state = ALM_ARMED;
            alarm_data[i].hlo_state = ALM_ARMED;
        }
        alarm_data[i].humid_lo &= 0x7f;
    }

    // Load backlight brightness, usb talker switch from eeprom
    backlight = eeprom_read(EEADR_LCD);
    usb_talker = eeprom_read(EEADR_USB_TALKER);

    // Load label
    for(i=0; i<NUM_ID; i++){
        for(j=0; j<8; j++){
            label[i][j] = eeprom_read(EEADR_LABEL_START + i*8 + j);
        }
    }

    if(usb_talker){
        USBDeviceInit(); // usb_device.c
    }

    // Enable global interrupt
    INTCONbits.GIE = 1;
    INTCONbits.PEIE = 1;

    glcd_init();
    glcd_clear(0x0);
    rx_init();
    glcd_backlight(backlight);
    beep(0);

    while(1){

        if(usb_talker){
            USBDeviceTasks();
            CDCTxService();
        }

        if(xtsk_25ms){
            xtsk_25ms = 0;
            // Read switch
            sw = ~((PORTB & 0xf0) >> 4);
            if(sw == swo){
                if(swcnt == 2){
                    // Single key action
                    if(sw & 0x01) { sw1_push(); display();}
                    if(sw & 0x02) { sw2_push(); display();}
                    if(sw & 0x04) { sw3_push(); display();}
                    if(sw & 0x08) { sw4_push(); display();}
                }
                if(swcnt > 40){
                    // Repetive key action
                    if(sw & 0x01 && swcnt & 0x04) { sw1_push(); display();}
                    if(sw & 0x02 && swcnt & 0x04) { sw2_push(); display();}
                    if(sw & 0x04 && swcnt & 0x04) { sw3_push(); display();}
                    if(sw & 0x08 && swcnt & 0x04) { sw4_push(); display();}
                }
                swcnt++;
            } else {
                swcnt = 0;
            }
            swo = sw;
        }

        if(xtsk_500ms){
            xtsk_500ms = 0;
            check_card();
            display();
        }

        if(xtsk_1m){
            xtsk_1m = 0;
            log_record();
        }

        // Read data
        rxdata.uldata = rx_getdata();
        if(rxdata.uldata != NO_DATA){
            curr_data[rxdata.id].vbatt = rxdata.vbatt + 16; // 1.6V offset
            curr_data[rxdata.id].temp  = (rxdata.data[0] & 0x80) ? (short)0xf000 : 0;
            curr_data[rxdata.id].temp |= ((short)rxdata.data[0] << 4);
            curr_data[rxdata.id].temp |= (short)((rxdata.data[1] >> 4) & 0x0f);
            curr_data[rxdata.id].humid  = ((short)(rxdata.data[1] & 0x0f)) << 8;
            curr_data[rxdata.id].humid |= (short)rxdata.data[2];
            curr_data[rxdata.id].rx_epoch = epoch;
            curr_data[rxdata.id].rx_hour = daytime.hour;
            curr_data[rxdata.id].rx_min = daytime.min;
            rx_epoch = epoch;

            if(usb_talker){
                talk_to_host(rxdata.id, &curr_data[rxdata.id]);
            }
            // Alarm check
            alarm_check(rxdata.id, &curr_data[rxdata.id]);
        }
    }
}

/******************************************************************************/
/* RTC set supplement functions                                               */
/******************************************************************************/
struct daytime_st daytime_tmp;

void set_year(unsigned char inc)
{
    if(inc){
        if(daytime_tmp.year < 30){
            daytime_tmp.year++;
        } else {
            daytime_tmp.year = 10;
        }
    } else {
        if(daytime_tmp.year > 10){
            daytime_tmp.year --;
        } else {
            daytime_tmp.year = 30;
        }
    }
}

void set_mon(unsigned char inc)
{
    if(inc){
        if(daytime_tmp.mon < 12){
            daytime_tmp.mon++;
        } else {
            daytime_tmp.mon = 1;
        }
    } else {
        if(daytime_tmp.mon > 1){
            daytime_tmp.mon --;
        } else {
            daytime_tmp.mon = 12;
        }
    }
}

void set_day(unsigned char inc)
{
    unsigned char daymon;
    daymon = daymonth[daytime_tmp.mon];
    if(daytime_tmp.year %4 == 0 && daytime_tmp.mon == 2){
        daymon = 29;
    }

    if(inc){
        if(daytime_tmp.day < daymon){
            daytime_tmp.day++;
        } else {
            daytime_tmp.day = 1;
        }
    } else {
        if(daytime_tmp.day > 1){
            daytime_tmp.day --;
        } else {
            daytime_tmp.day = daymon;
        }
    }
}

void set_hour(unsigned char inc)
{
    if(inc){
        if(daytime_tmp.hour < 23){
            daytime_tmp.hour++;
        } else {
            daytime_tmp.hour = 0;
        }
    } else {
        if(daytime_tmp.hour > 0){
            daytime_tmp.hour --;
        } else {
            daytime_tmp.hour = 23;
        }
    }
}

void set_min(unsigned char inc)
{
    if(inc){
        if(daytime_tmp.min < 59){
            daytime_tmp.min++;
        } else {
            daytime_tmp.min = 0;
        }
    } else {
        if(daytime_tmp.min > 0){
            daytime_tmp.min --;
        } else {
            daytime_tmp.min = 59;
        }
    }
}

void set_sec(unsigned char inc)
{
    if(inc){
        if(daytime_tmp.sec < 59){
            daytime_tmp.sec++;
        } else {
            daytime_tmp.sec = 0;
        }
    } else {
        if(daytime_tmp.sec > 0){
            daytime_tmp.sec --;
        } else {
            daytime_tmp.sec = 59;
        }
    }
}

/******************************************************************************/
/* Display / Key control                                                      */
/******************************************************************************/
enum dispid_e {
    DISPID_ID0, DISPID_ID1, DISPID_ID2, DISPID_ID3, DISPID_ID4,
    DISPID_ID5, DISPID_ID6, DISPID_ID7, DISPID_ID8, DISPID_ID9,
    DISPID_ALL, DISPID_LBL, DISPID_SDC, DISPID_STA, DISPID_SET, DISPID_END
} dispid = DISPID_SET;

unsigned char edit = 0;

enum {
    IDX_L0, IDX_L1, IDX_L2, IDX_L3, IDX_L4, IDX_L5, IDX_TMP_HI, IDX_TMP_LO, IDX_HUM_HI, IDX_HUM_LO, IDX_END
} edit_idx = IDX_TMP_HI;

enum {
    SET_YEAR, SET_MONTH, SET_DAY, SET_HOUR, SET_MIN, SET_SEC, SET_LCD, SET_USB, SET_END
} edit_set = SET_YEAR;

void sw1_push()
{
    alarm_mute();
    if(edit){
        if(dispid >= DISPID_ID0 && dispid <= DISPID_ID9){
            if(edit_idx < IDX_END -1){
                edit_idx ++;
            } else {
                edit_idx = 0;
            }
        }
        if(dispid == DISPID_SET){
            if(edit_set < SET_END -1){
                edit_set ++;
            } else {
                edit_set = 0;
            }
        }
        return;
    }

    // Display paging
    if(dispid > 0){
        dispid --;
    } else {
        dispid = DISPID_END -1;
    }
}

void sw2_push()
{
    alarm_mute();
    if(edit){
        if(dispid >= DISPID_ID0 && dispid <= DISPID_ID9){
            // Individual alarm edit here
            switch(edit_idx){
                case IDX_L0:
                case IDX_L1:
                case IDX_L2:
                case IDX_L3:
                case IDX_L4:
                case IDX_L5:
                    if(label[dispid][edit_idx] < 0x7e){
                       label[dispid][edit_idx]++;
                    }
                    break;

                case IDX_TMP_HI:
                    if(alarm_data[dispid].temp_hi < 50){
                        alarm_data[dispid].temp_hi++;
                    }
                    break;
                case IDX_TMP_LO:
                    if(alarm_data[dispid].temp_lo < 50){
                        alarm_data[dispid].temp_lo++;
                    }break;
                case IDX_HUM_HI:
                    if(alarm_data[dispid].humid_hi < 100){
                            alarm_data[dispid].humid_hi++;
                    }
                    break;
                case IDX_HUM_LO:
                    if(alarm_data[dispid].humid_lo < 100){
                        alarm_data[dispid].humid_lo++;
                    }
                    break;
            }
        }
        if(dispid == DISPID_SET){
            switch(edit_set){
                case SET_YEAR: set_year(1); break;
                case SET_MONTH: set_mon(1); break;
                case SET_DAY: set_day(1); break;
                case SET_HOUR: set_hour(1); break;
                case SET_MIN: set_min(1); break;
                case SET_SEC: set_sec(1); break;
                case SET_LCD: if(backlight < 9) backlight++; break;
                case SET_USB: usb_talker = 1; break;
            }
            glcd_backlight(backlight);
        }
        return;
    }

    // Display paging
    if(dispid < DISPID_END -1){
        dispid ++;
    } else {
        dispid = 0;
    }
}

void sw3_push()
{
    alarm_mute();
    if(edit){
        if(dispid >= DISPID_ID0 && dispid <= DISPID_ID9){
            // Individual alarm edit here
            switch(edit_idx){
                case IDX_L0:
                case IDX_L1:
                case IDX_L2:
                case IDX_L3:
                case IDX_L4:
                case IDX_L5:
                    if(label[dispid][edit_idx] > 0x20){
                       label[dispid][edit_idx]--;
                    }
                    break;
                case IDX_TMP_HI:
                    if(alarm_data[dispid].temp_hi > -20){
                        alarm_data[dispid].temp_hi--;
                    }
                    break;
                case IDX_TMP_LO:
                    if(alarm_data[dispid].temp_lo > -20){
                        alarm_data[dispid].temp_lo--;
                    }
                    break;
                case IDX_HUM_HI:
                    if(alarm_data[dispid].humid_hi > 0){
                        alarm_data[dispid].humid_hi--;
                    }
                    break;
                case IDX_HUM_LO:
                    if(alarm_data[dispid].humid_lo > 0){
                        alarm_data[dispid].humid_lo--;
                    }
                    break;
            }
        }
        if(dispid == DISPID_SET){
            switch(edit_set){
                case SET_YEAR: set_year(0); break;
                case SET_MONTH: set_mon(0); break;
                case SET_DAY: set_day(0); break;
                case SET_HOUR: set_hour(0); break;
                case SET_MIN: set_min(0); break;
                case SET_SEC: set_sec(0); break;
                case SET_LCD: if(backlight > 0) backlight--; break;
                case SET_USB: usb_talker = 0; break;
            }
            glcd_backlight(backlight);
        }
        return;
    }
    if(dispid >= DISPID_ID0 && dispid <= DISPID_ID9){
        if(alarm_data[dispid].thi_state == ALM_OFF){
            alarm_data[dispid].thi_state = ALM_ARMED;
            alarm_data[dispid].tlo_state = ALM_ARMED;
            alarm_data[dispid].hhi_state = ALM_ARMED;
            alarm_data[dispid].hlo_state = ALM_ARMED;
            eeprom_write(dispid *4 + EEADR_HUM_LO_OFS, alarm_data[dispid].humid_lo | 0x80);
        } else {
            alarm_data[dispid].thi_state = ALM_OFF;
            alarm_data[dispid].tlo_state = ALM_OFF;
            alarm_data[dispid].hhi_state = ALM_OFF;
            alarm_data[dispid].hlo_state = ALM_OFF;
            eeprom_write(dispid *4 + EEADR_HUM_LO_OFS, alarm_data[dispid].humid_lo);
        }
    }

    if(dispid == DISPID_SDC){
        log_start();
    }
    if(dispid == DISPID_STA){
        stat_count.good = 0;
        stat_count.overflow = 0;
        stat_count.premature = 0;
        stat_count.sumerr = 0;
        glcd_clear(0);
    }
}

void sw4_push()
{
    char i;
    alarm_mute();
    if(edit){
        if(dispid >= DISPID_ID0 && dispid <= DISPID_ID9){
            // Save alarm setting to eeprom
            eeprom_write(dispid *4 + EEADR_TMP_HI_OFS, alarm_data[dispid].temp_hi);
            eeprom_write(dispid *4 + EEADR_TMP_LO_OFS, alarm_data[dispid].temp_lo);
            eeprom_write(dispid *4 + EEADR_HUM_HI_OFS, alarm_data[dispid].humid_hi);
            i = alarm_data[dispid].hhi_state == ALM_OFF ? 0x00 : 0x80;
            eeprom_write(dispid *4 + EEADR_HUM_LO_OFS, alarm_data[dispid].humid_lo | i);

            // Save label to eeprom
            for(i=0; i<8; i++){
                eeprom_write( EEADR_LABEL_START + dispid *8 + i, label[dispid][i]);
            }

            edit = 0;
        }
        if(dispid == DISPID_SET){
            // Apply setup changes
            daytime_set(&daytime_tmp);
            // Save LCD brightness to eeprom
            eeprom_write(EEADR_LCD, backlight);
            eeprom_write(EEADR_USB_TALKER, usb_talker);
            edit = 0;
        }
        return;
    }

    if(dispid >= DISPID_ID0 && dispid <= DISPID_ID9){
        edit = 1;
    }

    if(dispid == DISPID_SDC){
        log_stop();
    }
    
    if(dispid == DISPID_SET){
        daytime_tmp = daytime;
        edit = 1;
    }
}

#define VLOBATT  20
#define TLIVE   600

unsigned char dispido;
void display()
{
    struct curr_data_st *cdp;
    struct alarm_data_st *adp;
    int i,j, blink, lobatt, bell, live;
    short temp;
    short humid;
    char polarity;
    struct daytime_st daytime_disp;

    if(daytime.sec & 0x01){
        blink = 1;
    } else {
        blink = 0;
    }

    if(dispid != dispido) glcd_clear(0);
    dispido = dispid;

    glcd_backlight(backlight);

    // Display header
    glcd_cursor(0,0);
    switch(dispid){
        case DISPID_ID0: case DISPID_ID1: case DISPID_ID2: case DISPID_ID3: case DISPID_ID4:
        case DISPID_ID5: case DISPID_ID6: case DISPID_ID7: case DISPID_ID8: case DISPID_ID9:
            tiny_printf(glcd_putc, NULL, "%2d:", dispid);
            for(i=0; i<6; i++){
                glcd_putc(label[dispid][i]);
            };
            if(edit && edit_idx < 6){
                glcd_reverse(0, 3 + edit_idx, 1);
            }
            break;
        case DISPID_ALL: glcd_puts("10:ALL"); break;
        case DISPID_LBL: glcd_puts("11:LABEL"); break;
        case DISPID_SDC: glcd_puts("12:SD"); break;
        case DISPID_STA: glcd_puts("13:STAT"); break;
        case DISPID_SET: glcd_puts("14:SETUP"); break;
    }

    // Batt / Bell icon
    lobatt = 0;
    bell = 0;
    for(i=0; i<NUM_ID; i++){
        cdp = &curr_data[i];
        live = (epoch - cdp->rx_epoch < TLIVE) ? 1 : 0;
        if(live && cdp->vbatt < VLOBATT){
            lobatt = 1;
        }
        adp = &alarm_data[i];
        if(adp->thi_state == ALM_TRIGD || adp->thi_state == ALM_MUTE ||
           adp->tlo_state == ALM_TRIGD || adp->tlo_state == ALM_MUTE ||
           adp->hhi_state == ALM_TRIGD || adp->hhi_state == ALM_MUTE ||
           adp->hlo_state == ALM_TRIGD || adp->hlo_state == ALM_MUTE){
            bell = 1;
        }
    }
    glcd_cursor(0, 10);
    if(blink == 0 && lobatt){
        glcd_puticon(ICON_BAT);
    } else if(bell){
        glcd_puticon(ICON_BEL);
    } else {
        glcd_puticon(ICON_NON);
    }
    // ANT icon
    glcd_cursor(0, 12);
    if(epoch - rx_epoch < 2){
        glcd_puticon(ICON_ANT);
    } else {
        glcd_puticon(ICON_NON);
    }
    // SD icon
    glcd_cursor(0, 14);
    if(sd_state == SD_BUSY){
        glcd_puticon(ICON_SDC);
    } else {
        glcd_puticon(ICON_NON);
    }

    // Display clock
    glcd_cursor(0,16);
    tiny_printf(glcd_putc, NULL, "%02d:%02d", daytime.hour, daytime.min);
    glcd_line(1,56, 129, 56);

    // Display area
    switch(dispid){
        // Display individual
        case DISPID_ID0: case DISPID_ID1: case DISPID_ID2: case DISPID_ID3: case DISPID_ID4:
        case DISPID_ID5: case DISPID_ID6: case DISPID_ID7: case DISPID_ID8: case DISPID_ID9:
            cdp = &curr_data[dispid];
            adp = &alarm_data[dispid];
            polarity = (cdp->temp < 0) ? '-' : ' ';
            temp = abs(cdp->temp);
            humid = cdp->humid;
            lobatt = (cdp->vbatt < VLOBATT) ? 1 : 0;
            live = (epoch - cdp->rx_epoch < TLIVE) ? 1 : 0;

            // Display temp and humidity at large font.
            glcd_cursor(2,1);
            if(live){
                if((alarm_data[dispid].thi_state == ALM_TRIGD || alarm_data[dispid].thi_state == ALM_MUTE ||
                    alarm_data[dispid].tlo_state == ALM_TRIGD || alarm_data[dispid].tlo_state == ALM_MUTE) && blink == 0){
                    tiny_printf(glcd_putc16, NULL, "     C  ");
                } else {
                    tiny_printf(glcd_putc16, NULL, "%c%2d.%01dC  ", polarity, temp/10, temp%10);
                }
                if((alarm_data[dispid].hhi_state == ALM_TRIGD || alarm_data[dispid].hhi_state == ALM_MUTE ||
                    alarm_data[dispid].hlo_state == ALM_TRIGD || alarm_data[dispid].hlo_state == ALM_MUTE) && blink == 0){
                    tiny_printf(glcd_putc16, NULL, "    %%");
                } else {
                    tiny_printf(glcd_putc16, NULL, "%2d.%01d%%", humid/10, humid%10);
                }
            } else {
                tiny_printf(glcd_putc16, NULL, "  -.-C   -.-%%");
            }

            // Display temp/humid Hi-Lo alarm setting.
             glcd_cursor(4,0);
            if(adp->hhi_state == ALM_OFF){
                glcd_puticon(ICON_NON);
            } else {
                glcd_puticon(ICON_BEL);
            }
            if(adp->hhi_state != ALM_OFF || edit){
                glcd_cursor(4,2);
                tiny_printf(glcd_putc, NULL, "%c%2d/%c%2d   %3d/%3d",
                    (adp->temp_hi < 0) ? '-' : ' ',
                    abs(adp->temp_hi),
                    (adp->temp_lo < 0) ? '-' : ' ',
                    abs(adp->temp_lo),
                    adp->humid_hi,
                    adp->humid_lo);
            } else {
                glcd_cursor(4,2);
                glcd_puts("                 ");
            }
            
            if(edit){
                switch(edit_idx){
                    case IDX_TMP_HI: glcd_reverse(4,2,3); break;
                    case IDX_TMP_LO: glcd_reverse(4,6,3); break;
                    case IDX_HUM_HI: glcd_reverse(4,12,3); break;
                    case IDX_HUM_LO: glcd_reverse(4,16,3); break;
                }
            }

            // Display RX time, Batt voltage
            glcd_cursor(5,1);
            tiny_printf(glcd_putc, NULL, "Rcv:%2d:%02d", cdp->rx_hour, cdp->rx_min);
            if(live && lobatt && blink == 1){
                tiny_printf(glcd_putc, NULL, "          ");
            } else {
                tiny_printf(glcd_putc, NULL, "  Bat:%01d.%01dV", cdp->vbatt/10, cdp->vbatt%10);
            }
            break;

        case DISPID_ALL:
            // Display all data at a glance
            glcd_cursor(1,3); tiny_printf(glcd_putc, NULL, "Temp Rh    Temp Rh");
            for(i=0; i<NUM_ID; i++){
                cdp = &curr_data[i];
                polarity = (cdp->temp < 0) ? '-' : ' ';
                temp = abs(cdp->temp);
                humid = cdp->humid;
                lobatt = (cdp->vbatt < VLOBATT) ? 1 : 0;
                live = (epoch - cdp->rx_epoch < TLIVE) ? 1 : 0;
                if(i%2 == 0){
                    glcd_cursor(2+i/2, 0);  // Left column
                } else {
                    glcd_cursor(2+i/2, 11); // Right column
                }
                tiny_printf(glcd_putc, NULL, "%d:", i);
                if(live) {
                    if(lobatt && blink == 0){
                        tiny_printf(glcd_putc, NULL, " Lo batt");
                    } else {
                        if((alarm_data[i].thi_state == ALM_TRIGD || alarm_data[i].thi_state == ALM_MUTE ||
                            alarm_data[i].tlo_state == ALM_TRIGD || alarm_data[i].tlo_state == ALM_MUTE) && blink == 0){
                            tiny_printf(glcd_putc, NULL, "      ");
                        } else {
                            tiny_printf(glcd_putc, NULL, "%c%2d.%01d ", polarity, temp/10, temp%10);
                        }
                        if((alarm_data[i].hhi_state == ALM_TRIGD || alarm_data[i].hhi_state == ALM_MUTE ||
                            alarm_data[i].hlo_state == ALM_TRIGD || alarm_data[i].hlo_state == ALM_MUTE) && blink == 0){
                            tiny_printf(glcd_putc, NULL, "  ");
                        } else {
                            tiny_printf(glcd_putc, NULL, "%2d", humid/10);
                        }
                    }
                } else {
                    tiny_printf(glcd_putc, NULL, "  -   - ");
                }
            }
            break;

        case DISPID_LBL: // ALL label
            for(i=0; i<NUM_ID; i++){
                cdp = &curr_data[i];
                lobatt = (cdp->vbatt < VLOBATT) ? 1 : 0;
                live = (epoch - cdp->rx_epoch < TLIVE) ? 1 : 0;
                if(i % 2 == 0){
                    glcd_cursor(2+i/2, 0);  // Left column
                } else {
                    glcd_cursor(2+i/2, 11); // Right column
                }
                tiny_printf(glcd_putc, NULL, "%d: ", i);
                if(live && lobatt && blink == 0){
                    tiny_printf(glcd_putc, NULL, "Lo batt");
                } else {
                    for(j=0; j<6; j++){
                        glcd_putc(label[i][j]);
                    };
                    glcd_putc(' ');
                }
            }
            break;

        case DISPID_SDC: // SD
            glcd_cursor(2,1);
            tiny_printf(glcd_putc, NULL, "Card: %s", sd_msg[sd_state]);
            glcd_cursor(3,1);
            tiny_printf(glcd_putc, NULL, " Err: %d", fs_lasterr);
            break;

        case DISPID_STA: // Stat
            glcd_cursor(2,1);
            tiny_printf(glcd_putc, NULL, "RxOK : %ld", stat_count.good);
            glcd_cursor(3,1);
            tiny_printf(glcd_putc, NULL, "RxErr: %ld(Prem)", stat_count.premature);
            glcd_cursor(4,1);
            tiny_printf(glcd_putc, NULL, "       %ld(Sum)", stat_count.sumerr);
            glcd_cursor(5,1);
            {
                long tmp = (epoch-1000) %86400;
                tiny_printf(glcd_putc, NULL, "Up   : %ldd %02d:%02d",
                    (epoch-1000) /86400,
                    (char)(tmp /3600), (char)((tmp %3600) /60));
            }
            break;
        case DISPID_SET:
            if(edit){
                daytime_disp = daytime_tmp;
            } else {
                daytime_disp = daytime;
            }
            glcd_cursor(2,1);
            tiny_printf(glcd_putc, NULL, "Date:%02d/%02d/%02d", daytime_disp.year, daytime_disp.mon, daytime_disp.day);
            glcd_cursor(3,1);
            tiny_printf(glcd_putc, NULL, "Time:%02d:%02d:%02d", daytime_disp.hour, daytime_disp.min, daytime_disp.sec);
            glcd_cursor(4,1);
            tiny_printf(glcd_putc, NULL, "Backlight: %d", backlight);
            glcd_cursor(5,1);
            tiny_printf(glcd_putc, NULL, "USB Talker: %s", usb_talker ? "on " : "off");
            if(edit){
                switch(edit_set){
                case SET_YEAR: glcd_reverse(2,6,2); break;
                case SET_MONTH: glcd_reverse(2,9,2); break;
                case SET_DAY: glcd_reverse(2,12,2); break;
                case SET_HOUR: glcd_reverse(3,6,2); break;
                case SET_MIN: glcd_reverse(3,9,2); break;
                case SET_SEC: glcd_reverse(3,12,2); break;
                case SET_LCD: glcd_reverse(4,12,1); break;
                case SET_USB: glcd_reverse(5,13,3); break;
                }
            }
            break;
    }

    // Key navigator
    glcd_line(1,8, 129, 8);
    glcd_cursor(7,0);
    switch(dispid){
        case DISPID_ID0: case DISPID_ID1: case DISPID_ID2: case DISPID_ID3: case DISPID_ID4:
        case DISPID_ID5: case DISPID_ID6: case DISPID_ID7: case DISPID_ID8: case DISPID_ID9:
            if(edit){
                glcd_puts(" Move Val+ Val- Done "); break;
            } else {
                glcd_puts(" Prev Next Alrm Edit "); break;
            }
        case DISPID_ALL:
        case DISPID_LBL:
            glcd_puts(" Prev Next"); break;
        case DISPID_STA:
            glcd_puts(" Prev Next Cler"); break;
        case DISPID_SDC:
            glcd_puts(" Prev Next Strt Stop "); break;
        case DISPID_SET:
            if(edit){
                glcd_puts(" Move Val+ Val- Done "); break;
            } else {
                glcd_puts(" Prev Next      Edit "); break;
            }
    }
}
// EOF

