/******************************************************************************/
/*  Wireless Temp/Humidity Logger                                             */
/*  Reciever unit data reciever                                               */
/*  Copyright(c) 2013 Hirofumi Momose  hirofumi[at]momose.com                 */
/******************************************************************************/

#include <xc.h>
#include "rx.h"

// Data queue
#define BUFF_SIZE 4
unsigned long buff[BUFF_SIZE];
unsigned char buff_ridx;
unsigned char buff_widx;
unsigned char buff_flags;
#define BUFF_FULL        (0x01)
#define BUFF_EMPTY       (0x02)
#define BUFF_OVERFLOW    (0x04)

#if 0   // debugging led
#define Toggle_RE0()    (LATEbits.LATE0 ^= 0xff)
#define Toggle_RE1()    (LATEbits.LATE1 ^= 0xff)
#define RE1_ON          (LATEbits.LATE1 = 1)
#define RE1_OFF         (LATEbits.LATE1 = 0)
#endif

// Stat counter
struct stat_count_st stat_count;

// Internal variables */

enum state_en {
    ST_0,       // Delay next frame
    ST_1,       // Bit sync
    ST_2,       // Watching Star Frame Delimiter
    ST_3        // Acquiring payload, sum
} state;

unsigned short bit_period_quad;
unsigned short bit_period_half;
unsigned char  nbit, ngood;
unsigned char  bit_fore, bit_back, bit_this;
union data_u   pay_data;
unsigned char  pay_sum;
unsigned char isforebit;
unsigned short tmr3_value_center, tmr3_value_lower, tmr3_value_upper;

void rx_init(void)
{
    // Init data queue
    buff_ridx = 0;
    buff_widx = 0;
    buff_flags = BUFF_EMPTY;
    
    // Setup port
    TRISBbits.TRISB2 = 1;
    
    // Setup timer3
    T3CONbits.TMR3CS = 0;   // Use 1/4 internal clock
    T3CONbits.T3CKPS1 = 1;  // Use 1/8 prescaler
    T3CONbits.T3CKPS0 = 1;  // Thus, TMR3 clock is 1.5MHz
    T3CONbits.T3SYNC = 1;
    T3CONbits.TMR3ON = 1;

    // Prepare for state0
    state = ST_0;
    INTCON3bits.INT2IE = 0;
    INTCON3bits.INT2IF = 0;
    PIR2bits.TMR3IF = 0;
    PIE2bits.TMR3IE = 1;
    TMR3 = 0xffff - BIT_PERIOD_MIN;
}

void rx_int(void)
{
    unsigned short tmr3_val;
    unsigned char data_sum;

    switch(state){
        case ST_0:
            TMR3 = BIT_PERIOD_MAX;
            PIE2bits.TMR3IE = 0;
            INTCON3bits.INT2IF = 0;
            INTCON3bits.INT2IE = 1;
            nbit = 0;
            ngood = 0;
            bit_period_quad = 0;
            state = ST_1;
            break;

        case ST_1:
            tmr3_val = TMR3;
            TMR3 = 0;
            INTCON3bits.INT2IF = 0;
            // Toggle_RE1();
            if(tmr3_val < BIT_PERIOD_MIN){
                if(ngood >0) ngood--;
                if(ngood == 0){
                    // Toggle_RE0();
                    TMR3 = 0xffff - BIT_PERIOD_MIN;
                    PIE2bits.TMR3IE = 1;
                    INTCON3bits.INT2IE = 0;
                    state = ST_0;
                    break;
                }
                TMR3 += tmr3_val;
                break;
            }
            if(tmr3_val > BIT_PERIOD_MAX){
                if(ngood >0) ngood--;
                nbit = 0;
                bit_period_quad = 0;
                break;
            }
            if(nbit > 3){
                bit_period_quad += tmr3_val;
                if(nbit == 7){
                    // TBD Bit period is determined 4 cycles (4 thru 7)
                    // Is it enought?
                    bit_period_half = bit_period_quad /8;
                    nbit =0;
                    ngood =0;
                    isforebit = 1;
                    TMR3 = 0xffff - bit_period_half /2;
                    PIE2bits.TMR3IE = 1;
                    // INTCON3bits.INT2IE = 0;
                    state = ST_2;
                    // Added below
                    tmr3_value_center = 0xffff - bit_period_half /2;
                    tmr3_value_lower  = 0xffff - (unsigned short)((bit_period_half /2) *1.2);
                    tmr3_value_upper  = 0xffff - (unsigned short)((bit_period_half /2) *0.8);
                    break;
                }
            }
            nbit++;
            ngood++;
            break;

        case ST_2:
            if(INTCON3bits.INT2IF){
                tmr3_val = TMR3;
                if(tmr3_val > tmr3_value_lower && tmr3_val < tmr3_value_upper){
                    // Data rising edge is within expected time window
                    TMR3 = tmr3_value_center;
                }
                INTCON3bits.INT2IF = 0;
                break;
            }
            // Toggle_RE0();
            TMR3 += 0xffff - bit_period_half;
            PIR2bits.TMR3IF = 0;
            if(isforebit){
                bit_fore = PORTBbits.RB2;
                isforebit = 0;
            } else {
                bit_back = PORTBbits.RB2;
                isforebit = 1;
                if(bit_fore == 1 && bit_back == 1){
                    nbit = 0;
                    pay_data.uldata = 0;
                    pay_sum = 0;
                    state = ST_3;
                    break;
                }
                if(nbit > MAX_PREAMBLE_BITS){
                    TMR3 = 0xffff - BIT_PERIOD_MIN;
                    state = ST_0;
                    stat_count.premature++;
                    break;
                }
                nbit ++;
            }
            break;

        case ST_3:
            if(INTCON3bits.INT2IF){
                tmr3_val = TMR3;
                if(tmr3_val > tmr3_value_lower && tmr3_val < tmr3_value_upper){
                    // Data rising edge is within expected time window
                    TMR3 = tmr3_value_center;
                }
                INTCON3bits.INT2IF = 0;
                break;
            }
            // Toggle_RE0();
            TMR3 += 0xffff - bit_period_half;
            PIR2bits.TMR3IF = 0;
            if(isforebit){
                bit_fore = PORTBbits.RB2;
                isforebit = 0;
            } else {
                bit_back  = PORTBbits.RB2;
                isforebit = 1;
                if(bit_fore == 0 && bit_back == 1){
                    // Rising edge found.
                    bit_this = 1;
                } else if(bit_fore == 1 && bit_back == 0){
                    // Falling edge found.
                    bit_this = 0;
                } else {
                    // Bit error
                    TMR3 = 0xffff - BIT_PERIOD_MIN;
                    state = ST_0;
                    stat_count.premature++;
                    break;
                }
                if(nbit < 32){
                    pay_data.uldata <<= 1;
                    pay_data.uldata |= bit_this;
                } else {
                    pay_sum <<= 1;
                    pay_sum |= bit_this;
                    if(nbit == 39){
                        // ALL bits recvd

                        // Check Sum
                        data_sum  = pay_data.c1 + pay_data.c2
                                  + pay_data.c3 + pay_data.c4;
                        if(data_sum == pay_sum){
                            // Sum OK. Push data to buffer
                            if(buff_flags & BUFF_FULL){
                                buff_flags |= BUFF_OVERFLOW;
                                stat_count.overflow++;
                            } else {
                                buff[buff_widx] = pay_data.uldata;
                                buff_flags &= ~BUFF_EMPTY;
                                if( buff_widx == buff_ridx -1
                                 || buff_widx == buff_ridx + BUFF_SIZE -1){
                                    // Buffer is now full state.
                                    buff_flags |= BUFF_FULL;
                                }
                                if(buff_widx == BUFF_SIZE -1){
                                    buff_widx = 0;
                                } else {
                                    buff_widx ++;
                                }
                            }
                            stat_count.good++;
                        } else {
                            // Sum error
                            stat_count.sumerr++;
                        }

                        // Back to initial state
                        TMR3 = 0xffff - BIT_PERIOD_MIN;
                        state = ST_0;
                        break;
                    }
                }
                nbit++;
                break;
            }
            break;
    }
}

unsigned long rx_getdata(void)
{
    unsigned long data;
    if(buff_flags & BUFF_EMPTY){
        return NO_DATA;
    }
    INTCONbits.GIE = 0;
    data = buff[buff_ridx];
    buff_flags &= ~BUFF_FULL;
    if(buff_ridx == buff_widx -1
    || buff_ridx == buff_widx + BUFF_SIZE -1){
        // Buffer is now empty state.
        buff_flags |= BUFF_EMPTY;
    }
    if(buff_ridx == BUFF_SIZE -1){
        buff_ridx = 0;
    } else {
        buff_ridx ++;
    }
    INTCONbits.GIE = 1;
    return data;
}
// EOF
