/******************************************************************************/
/*  Wireless Temp/Humidity Logger                                             */
/*  Reciever unit spi layer control                                           */
/*  Copyright(c) 2013 Hirofumi Momose  hirofumi[at]momose.com                 */
/******************************************************************************/

#include <xc.h>
#include "spi.h"

/**************************************************************************
 * SPI layer functions
 * spi_init()
 * spi_trx()
 * spi_cs_assert()      macro
 * spi_cs_deassert()    macro
 **************************************************************************/

// Initialize with low clock
void spi_init(void)
{
    // Setup port direction
    TRISCbits.TRISC7 = 0;   // Use RC7 as SDO (OUT)
    TRISBbits.TRISB0 = 1;   // Use RB0 as SDI (IN)
    TRISBbits.TRISB1 = 0;   // Use RB1 as SCL (OUT)
    TRISEbits.TRISE2 = 0;   // Use RE2 as CS  (OUT)

    // Other card info signals
    TRISCbits.TRISC6 = 1;   // Write protect  (IN)
    TRISBbits.TRISB3 = 1;   // Card detection (IN)

    // Use TMR2 to prescale spi clock
    // 100kHz - 400kHz clock is required at SPI initialization
    // Prepare 200kHz clock(400kHz /2) from TMR2
    T2CONbits.T2CKPS = 0;   // Use 1:1 prescaler
    PR2 = 30;
    TMR2ON = 1;

    SSPCON1bits.SSPEN = 0;  // SSPEN = 0
    SSPSTATbits.SMP = 1;    // SMP = 1
    SSPCON1bits.CKP = 1;    // CKP = 1;   // Idle state for clock is h level
    SSPSTATbits.CKE = 1;    // CKE = 1;   // Transmit occurs on transition from Idle to active clock state
    SSPCON1bits.SSPM = 3;   // Select master clock from TMR2/2
    SSPCON1bits.SSPEN = 1;  // SSPEN = 1; // Must be set after all SPI configs are done.
}

// Go full speed
void spi_fullspeed(void)
{
    SSPCON1bits.SSPEN = 0;  // Disable SPI
    SSPCON1bits.SSPM = 0;   // Select master clock from Fosc/4 (12MHz)
    SSPCON1bits.SSPEN = 1;  // Re-enable SPI
}

// Transmit / Recv
unsigned char spi_trx(unsigned char tx_data)
{
    SSPBUF = tx_data;
    while(SSPSTATbits.BF == 0) continue;
    return SSPBUF;
}
// EOF
